unit UnitThreads;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, RYFormulaEvaluator,
  StdCtrls, ExtCtrls;

type
  TComputationThread = class(TThread)
  private
    aExpr: TRyFormulaEval;
    procedure FindVar(VariableName: string; var Value: Extended);
  public
    LastResult: Extended;
    procedure Execute; override;
    constructor Create(Susp: Boolean);
    destructor Destroy; override;
  end;

  TForm2 = class(TForm)
    Button1: TButton;
    Label1: TLabel;
    Lblnum: TLabel;
    Memo1: TMemo;
    Timer1: TTimer;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    { Private declarations }
    l: TList;
    Num: Integer;
  public
    { Public declarations }
  end;

var
  Form2: TForm2;

implementation

{$R *.DFM}

{ TComputationThread }

constructor TComputationThread.Create(Susp: Boolean);
begin
  inherited Create(True);
  aExpr := TRyFormulaEval.Create;
  aExpr.UseStandardFunctions;
  aExpr.OnFindVariable := FindVar;
  aExpr.Formula := 'MAX(x, y) * MIN(k*l+o, u)';
  aExpr.Parse;
  Priority := tpNormal;
end;

destructor TComputationThread.Destroy;
begin
  aExpr.Free;
  inherited;
end;

procedure TComputationThread.Execute;
begin
  while not Terminated do
  begin
    LastResult := aExpr.Result;
    sleep(20);
  end;
end;

procedure TComputationThread.FindVar(VariableName: string;
  var Value: Extended);
begin
  if VariableName = 'x' then
    Value := 10
  else   if VariableName = 'y' then
    Value := Random(2000)
  else   if VariableName = 'k' then
    Value := 30.67
  else   if VariableName = 'l' then
    Value := 234
  else   if VariableName = 'o' then
    Value := 567
  else   if VariableName = 'u' then
    Value := 2.245;
end;

procedure TForm2.Button1Click(Sender: TObject);
var
  ct: TComputationThread;
begin
  ct := TComputationThread.Create(True);
  ct.Resume;
  l.Add(ct);
  Inc(Num);
  LblNum.Caption := IntToStr(Num);
end;

procedure TForm2.FormCreate(Sender: TObject);
begin
  l := TList.Create;
  Num := 0;
end;

procedure TForm2.FormDestroy(Sender: TObject);
var
 t: TThread;
begin
 while l.Count > 0 do
 begin
   t := l.Items[0];
   t.Terminate;
   t.WaitFor;
   t.Free;
   l.Delete(0);
 end;
  l.Free;
end;

procedure TForm2.Timer1Timer(Sender: TObject);
var
  k: Integer;
  t: TComputationThread;
begin
  if Assigned(l) and (l.Count > 0) then
  begin
    if Memo1.Lines.Count > 200 then
      Memo1.Lines.Clear;
    Memo1.Lines.Add('-----------------------------------------');
    for k := 0 to l.Count-1 do
    begin
      t := l.Items[k];
      Memo1.Lines.Add('THR'+ Format('%4.4d=%f', [k, t.LastResult]));
    end;
    Memo1.Lines.Add('-----------------------------------------');
  end;
end;

end.
 